from Products.Five.browser import pagetemplatefile
from plone.app.form._named import named_template_adapter
from zope.formlib import namedtemplate, interfaces
from zope import i18n

from plone.app.form import _patches
_patches.apply_patches()

@namedtemplate.implementation(interfaces.IAction)
def render_submit_button(self):
    """A custom version of the submit button that uses plone's context class"""
    if not self.available():
        return ''
    label = self.label
    if isinstance(label, i18n.Message):
        label = i18n.translate(self.label, context=self.form.request)
    return ('<input type="submit" id="%s" name="%s" value="%s"'
            ' class="context" />' %
            (self.__name__, self.__name__, label)
            )

__all__ = ('named_template_adapter', 'default_named_template')

_template = pagetemplatefile.ViewPageTemplateFile('pageform.pt')
default_named_template_adapter = named_template_adapter(_template)

_subpage_template = pagetemplatefile.ViewPageTemplateFile('subpageform.pt')
default_subpage_template = named_template_adapter(_subpage_template)

_template = pagetemplatefile.ViewPageTemplateFile('addingpageform.pt')
adding_named_template_adapter = named_template_adapter(_template)


#
from checkboxwidget import CheckBoxWidget
from checkboxwidget import DisabledCheckBoxWidget
from multicheckboxwidget import MultiCheckBoxWidget
from selectionwidget import LanguageDropdownChoiceWidget



"""Event definitions
"""

from zope.interface import implements

from zope.component.interfaces import ObjectEvent

from plone.app.form.interfaces import IEditBegunEvent
from plone.app.form.interfaces import IEditCancelledEvent
from plone.app.form.interfaces import IEditSavedEvent

class EditBegunEvent(ObjectEvent):
    """An edit operation was begun
    """
    implements(IEditBegunEvent)
    
class EditCancelledEvent(ObjectEvent):
    """An edit operation was cancelled
    """
    implements(IEditCancelledEvent)
    
class EditSavedEvent(ObjectEvent):
    """An edit operation was completed
    """
    implements(IEditSavedEvent)


from Products.PloneTestCase import PloneTestCase as ptc
from Products.CMFCore.utils import getToolByName

from zope.publisher.browser import TestRequest
from zope.app.component.hooks import getSite

from plone.app.form.widgets.wysiwygwidget import WYSIWYGWidget

ptc.setupPloneSite()


class WYSIWYGWidgetTestCase(ptc.PloneTestCase):
    """Base class used for test cases
    """

    def test_right_macro(self):
        # fixes #8016
        class MyField:
            __name__ = 'the field'
            required = True
            default = u'the value'
            missing_value = None

        # the wysiwyg widget depends on the used editor
        pm = getToolByName(self.portal, 'portal_membership')
        member = pm.getAuthenticatedMember()
        editor = member.getProperty('wysiwyg_editor', '').lower()

        # we have kupu by default
        self.assertEquals(editor, 'kupu')

        # so it means the widget should use the macro
        # provided by kupu (default skin with .css includes)
        w = WYSIWYGWidget(MyField(), TestRequest())
        kupu = w()

        # let's add a custom editor
        # with a fake skin that should be catched
        # to provide a custom macro
        site = getSite()
        class  MyMacros(object):
            def wysiwygEditorBox(self):
                return (('version', '1.6'), ('mode', 'html'))
        class MySkin(object):
            macros = MyMacros()
        site.cool_editor_wysiwyg_support = MySkin()

        # let's change it to `cool_editor`
        member.setMemberProperties({'wysiwyg_editor': 'cool_editor'})

        w = WYSIWYGWidget(MyField(), TestRequest())
        cool_editor = w()

        # the macro used by wysiwygwidget should differ
        self.assertNotEquals(kupu, cool_editor)

def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(WYSIWYGWidgetTestCase))
    return suite



from zope.app.form.browser.widget import renderElement
from zope.i18nmessageid import MessageFactory
from zope.i18n import translate
from zope.app.form.browser import CheckBoxWidget as BaseWidget

_ = MessageFactory('plone')

class CheckBoxWidget(BaseWidget):
    """ Plone specific widget that is going to show the checkbox widget on the left of the label
        in order to do that we remove the title / label / required
    """

    def __init__(self, context, request):
        BaseWidget.__init__(self, context, request)
        self.required = self.__required = False
        self.__name = self.name
        self.name = ""

    label = property(lambda self: "")

    hint = property(lambda self: "")

    disabled = False

    def error(self):
        return ""

    def __call__( self ):
        """Render the widget to HTML."""
        value = self._getFormValue()
        html = "<label for='%s'>%s</label>\n" % (self.name , translate(super(BaseWidget,self).label, context=self.request))
        if self.__required:
            html += "<span class='fieldRequired' title='%s'>%s</span>" % (translate(_(u'title_required', default='Required'), context=self.request), translate(_(u'label_required', default='(Required)'), context=self.request))
        if super(BaseWidget, self).hint:
            html += "<div class='formHelp'>%s</div>" % translate(super(BaseWidget, self).hint, context=self.request)
        if super(BaseWidget, self).error() != '':
            html += "<div>%s</div>" % translate(super(BaseWidget, self).error(), context=self.request)
        
        if value == 'on':
            kw = {'checked': 'checked'}
        else:
            kw = {}
        if self.disabled:
            kw['disabled'] = 'disabled'
        return "%s  %s %s" % (
            renderElement(self.tag,
                          type='hidden',
                          name=self.name+".used",
                          id=self.name+".used",
                          value=""
                          ),
            renderElement(self.tag,
                          type=self.type,
                          name=self.name,
                          id=self.name,
                          cssClass=self.cssClass,
                          extra=self.extra,
                          value="on",
                          **kw),
            html
            )


class DisabledCheckBoxWidget(CheckBoxWidget):
    """Simple variation of the CheckBoxWidget which renders itself disabled.
    """
    
    disabled = True



import unittest
import Products.Five
import plone.app.form
from zope.testing import doctest, cleanup
from Products.Five import zcml

optionflags =  (doctest.ELLIPSIS |
                doctest.NORMALIZE_WHITESPACE |
                doctest.REPORT_ONLY_FIRST_FAILURE)


def setUp(test):
    zcml.load_config('configure.zcml', Products.Five)
    zcml.load_config('configure.zcml', plone.app.form)

def tearDown(test):
    cleanup.cleanUp()

def test_suite():
    return unittest.TestSuite([
        doctest.DocFileSuite('widgets/uberselectionwidget.txt',
                             package='plone.app.form',
                             setUp=setUp,
                             tearDown=tearDown,
                             optionflags=optionflags),
        ])



__docformat__ = 'restructuredtext'

from cgi import escape

from zope.interface import implements
from zope.i18n import translate

from zope.app.form.interfaces import IWidgetInputError
from zope.app.form.browser.interfaces import IWidgetInputErrorView

class WidgetInputErrorView(object):
    """A widget error view that hardcodes no html"""
    implements(IWidgetInputErrorView)

    __used_for__ = IWidgetInputError

    def __init__(self, context, request):
        self.context, self.request = context, request

    def snippet(self):
        """Convert a widget input error to an html snippet

        >>> from zope.app.form.interfaces import WidgetInputError
        >>> class TooSmallError(object):
        ...     def doc(self):
        ...         return "Foo input < 1"
        >>> err = WidgetInputError("foo", "Foo", TooSmallError())
        >>> view = WidgetInputErrorView(err, None)
        >>> view.snippet()
        u'Foo input &lt; 1'

        The only method that IWidgetInputError promises to implement is
        `doc()`. Therefore, other implementations of the interface should also
        work.

        >>> from zope.app.form.interfaces import ConversionError
        >>> err = ConversionError('Could not convert to float.')
        >>> view = WidgetInputErrorView(err, None)
        >>> view.snippet()
        u'Could not convert to float.'
        """
        message = self.context.doc()
        translated = translate(message, context=self.request, default=message)
        return u'%s' % escape(translated)



from zope.app.form.browser.textwidgets import TextWidget
from zope.app.pagetemplate.viewpagetemplatefile import ViewPageTemplateFile

from Acquisition import aq_parent
from Products.CMFCore.interfaces import ISiteRoot
from zope.app.component.hooks import getSite

class WYSIWYGWidget(TextWidget):
    """ A widget using Plone's wysiwyg_support.
    """

    template = ViewPageTemplateFile('wysiwygwidget.pt')

    def __init__(self, field, request):
        super(TextWidget, self).__init__(field, request)

    def __call__(self):
        value = self._getFormValue()
        if value is None or value == self.context.missing_value:
            value = ''

        # Evil acquisition majik to find the site root. This is made tricky
        # by the fact that the widget doesn't have a direct path to its
        # context (you can go self.context.context, but this may be an
        # adapter on the context, not the context itself.). We can find
        # the root using getUtility(ISiteRoot), but this isn't wrapped
        # in the request container. We can use getSite(), but there may be
        # other sites in-between, not at least the KSS site-in-a-view. Sigh.

        site = getSite()
        while site is not None and not ISiteRoot.providedBy(site):
            site = aq_parent(site)
        
        return self.template(form_context=site,
                             value=value)


from zope.component import getMultiAdapter
from zope.formlib import form as formlib

from kss.core import kssaction
from plone.app.kss.plonekssview import PloneKSSView

from Acquisition import aq_inner

class FormlibValidation(PloneKSSView):
    """KSS actions for formlib form inline validation
    """

    @kssaction
    def validate_input(self, formname, fieldname, value=None):
        """Given a form (view) name, a field name and the submitted
        value, validate the given field.
        """
        
        # Abort if there was no value changed. Note that the actual value
        # comes along the submitted form, since a widget may require more than
        # a single form field to validate properly.
        if value is None:
            return
        
        context = aq_inner(self.context)
        request = aq_inner(self.request)
        
        # Find the form, the field and the widget

        form = getMultiAdapter((context, request), name=formname)
        form = form.__of__(context)
        
        raw_fieldname = fieldname[len(form.prefix)+1:]
        formlib_field = form.form_fields[raw_fieldname]
 
        widgets = formlib.setUpWidgets((formlib_field,), form.prefix, context, 
            request, form=form, adapters={}, ignore_request=False)
            
        widget = widgets[raw_fieldname]
        
        # Attempt to convert the value - this will trigge validation
        ksscore = self.getCommandSet('core')
        kssplone = self.getCommandSet('plone')
        validate_and_issue_message(ksscore, widget, fieldname, kssplone)
            
def validate_and_issue_message(ksscore, widget, fieldname, kssplone=None):
    """A helper method also used by the inline editing view
    """

    error = None
    try:
        widget.getInputValue()
    except:
        pass
    error = widget.error()

    field_div = ksscore.getHtmlIdSelector('formfield-%s' % fieldname.replace('.', '-'))
    error_box = ksscore.getCssSelector('#formfield-%s div.fieldErrorBox' % fieldname.replace('.', '-'))
    
    if error:
        ksscore.replaceInnerHTML(error_box, error)
        ksscore.addClass(field_div, 'error')
    else:
        ksscore.clearChildNodes(error_box)
        ksscore.removeClass(field_div, 'error')
        if kssplone is not None: 
            kssplone.issuePortalMessage('')

    return bool(error)



from xml.sax.saxutils import unescape

from zope.component import getMultiAdapter
from zope.event import notify

from zope import lifecycleevent
from zope.app.form.interfaces import IDisplayWidget
from zope.formlib import form as formlib

from kss.core import kssaction
from plone.app.kss.plonekssview import PloneKSSView

from Acquisition import aq_inner
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile

from plone.locking.interfaces import ILockable

from plone.app.form.kss.validation import validate_and_issue_message

class FormlibInlineEdit(PloneKSSView):
    """KSS actions for formlib form inline editing
    """

    form_template = ViewPageTemplateFile('inline_edit_wrapper.pt')

    @kssaction
    def begin(self, formname, fieldname, structure='false'):
        """Begin inline editing - find the widget for the given field name
        in the given form (looked up as a view on the context), then hide the
        block with the id '${fieldname}-display' and display an edit form in
        its place. If 'structure' is 'true' (a string), then the inline 
        editable field will eventually permit HTML input to be rendered
        unescaped.
        """
        context = aq_inner(self.context)
        request = aq_inner(self.request)
        
        form = getMultiAdapter((context, request), name=formname)
        form = form.__of__(context)
        
        if fieldname.startswith(form.prefix):
            fieldname = fieldname[len(form.prefix)+1:]
            
        formlib_field = form.form_fields[fieldname]
        widgets = formlib.setUpEditWidgets((formlib_field,), form.prefix, 
            context, request, ignore_request=True)
            
        widget = widgets[fieldname]
        
        display_id = '%s-display' % fieldname
        form_id = '%s-form' % fieldname
        
        ksscore = self.getCommandSet('core')
        zopecommands = self.getCommandSet('zope')
        plonecommands = self.getCommandSet('plone')
        
        # lock the context (or issue warning)
        locking = ILockable(context, None)
        if locking:
            if not locking.can_safely_unlock():
                selector = ksscore.getHtmlIdSelector('plone-lock-status')
                zopecommands.refreshViewlet(selector, 'plone.abovecontent', 'plone.lockinfo')
                plonecommands.refreshContentMenu()
                return
            else: # we are locking the content
                locking.lock()
        
        plonecommands.issuePortalMessage('')
        
        # hide the existing display field
        display_selector = ksscore.getHtmlIdSelector(display_id)
        ksscore.addClass(display_selector, 'hiddenStructure')
        
        # show the form
        form_html = self.form_template(widget=widget,
                                       form_id=form_id,
                                       fieldname=fieldname,
                                       structure=structure)

        ksscore.insertHTMLAfter(display_selector, form_html)
        
        # XXX: Focus on the input field?
        
    @kssaction
    def cancel(self, fieldname):
        """Cancel the inline editing taking place for the given field, by
        removing the inline editing form and unhiding the block with id
        '${fieldname}-display'.
        """
        context = aq_inner(self.context)
        
        display_id = '%s-display' % fieldname
        form_id = '%s-form' % fieldname
        
        ksscore = self.getCommandSet('core')
        
        # unlock the context if it was locked before
        locking = ILockable(context, None)
        if locking and locking.can_safely_unlock():
            locking.unlock()

        # show the existing display field
        ksscore.removeClass(ksscore.getHtmlIdSelector(display_id), 'hiddenStructure')
        
        # hide the form
        ksscore.deleteNode(ksscore.getHtmlIdSelector(form_id))
        
    @kssaction
    def save(self, formname, fieldname, structure='false'):
        """Attempt to save the given field in the given form which is being
        inline-edited. If there is a validation error, the error will be
        highlighted. If not, the value will be saved, and the inline editing
        form removed. Before the form is re-displayed, the contents of the
        element with id '${fieldname}-display' will be updated with the new
        field value.
        """
        
        context = aq_inner(self.context)
        request = aq_inner(self.request)
        
        structure = (structure == 'true')
        
        form = getMultiAdapter((context, request), name=formname)
        form = form.__of__(context)
        
        raw_fieldname = fieldname
        if fieldname.startswith(form.prefix):
            raw_fieldname = fieldname[len(form.prefix)+1:]
            full_fieldname = fieldname
        else:
            full_fieldname = "%s.%s" % (form.prefix, raw_fieldname)
            
        formlib_field = form.form_fields[raw_fieldname]
        widgets = formlib.setUpEditWidgets((formlib_field,), form.prefix, 
            context, request, ignore_request=False)
            
        widget = widgets[raw_fieldname]
        
        # validate and issue a message if there's an error
        
        ksscore = self.getCommandSet('core')
        error = validate_and_issue_message(ksscore, widget, full_fieldname)

        if not error:
        
            field_value = request.form.get('%s.%s' % (form.prefix, raw_fieldname))
            data = { raw_fieldname : field_value }
        
            # update value
            changed = formlib.applyChanges(context, (formlib_field,), data)
            if changed:
                
                # send modified events if something actually changed
                field = widget.context
                adapter = field.interface(context)
                
                descriptor = lifecycleevent.Attributes(field.interface, raw_fieldname)
                notify(lifecycleevent.ObjectModifiedEvent(context, descriptor))
                
                # update the rendered value
                display_widget = getMultiAdapter((formlib_field.field, request), IDisplayWidget)
                display_widget.setRenderedValue(field.get(adapter))
                output = display_widget()
                
                if structure:
                    output = unescape(output)
                
                display_id = '%s-display' % fieldname
                ksscore.replaceInnerHTML(ksscore.getHtmlIdSelector(display_id), output)
        
            # unlock and remove the field
            self.cancel(fieldname)


def update_only_validator(form, action):
    """Only validate an action when updating a form.

    This allows you to create an action without having formlib render a
    button for it.
    """ 
    return "form_result" not in form.__dict__


def null_validator(*args, **kwargs):
    """A validator that doesn't validate anything.
    
    This is somewhat lame, but if you have a "Cancel" type button that
    won't want to validate the form, you need something like this.

    @form.action(_(u"label_cancel", default=u"Cancel"),
                 validator=null_validator,
                 name=u'cancel')
    """
    return ()



from zope.interface import Interface, Attribute
try:
    from zope.component.interfaces import IObjectEvent
except ImportError:
    from zope.app.event.interfaces import IObjectEvent
    
from zope.formlib.interfaces import IPageForm
from zope.formlib.interfaces import ISubPageForm

class IPlonePageForm(IPageForm):
    """A page form with a couple extra attributes
    """
    description = Attribute("A longer description to display on the form")
    form_name = Attribute("A label to apply to the fieldset")

class IPloneSubPageForm(ISubPageForm):
    """A page form with a couple extra attributes
    """
    description = Attribute("A longer description to display on the form")
    form_name = Attribute("A label to apply to the fieldset")

class IEditForm(Interface):
    """Marker interface for edit forms. This allows things like the locking
    widget to be registered for edit forms only.
    """
    
class IEditBegunEvent(IObjectEvent):
    """An event signalling that editing has begun on an object
    """
    
class IEditFinishedEvent(IObjectEvent):
    """Base event signalling that an edit operation has completed
    """
    
class IEditCancelledEvent(IEditFinishedEvent):
    """An event signalling that editing was cancelled on the given object
    """
    
class IEditSavedEvent(IEditFinishedEvent):
    """An event signalling that editing was complated on the given object
    """



from zope.app.form.browser.widget import renderElement
from zope.app.form.browser import MultiCheckBoxWidget as BaseWidget


class MultiCheckBoxWidget(BaseWidget):
    """Provide a list of checkboxes that provide the choice for the list,
       with a <label> for accessibility"""

    orientation = "vertical"

    _joinButtonToMessageTemplate = u"%s %s"
    
    def renderItem(self, index, text, value, name, cssClass):
        id = '%s.%s' % (name, index)
        elem = renderElement('input',
                             type="checkbox",
                             cssClass=cssClass,
                             name=name,
                             id=id,
                             value=value)

        label = renderElement('label',
                              extra= u"for=%s" % id,
                              contents=text)

        return self._joinButtonToMessageTemplate %(elem, label)

    def renderSelectedItem(self, index, text, value, name, cssClass):
        id = '%s.%s' % (name, index)
        elem = renderElement('input',
                             type="checkbox",
                             cssClass=cssClass,
                             name=name,
                             id=id,
                             value=value,
                             checked="checked")

        label = renderElement('label',
                              extra= u"for=%s" % id,
                              contents=text)

        return self._joinButtonToMessageTemplate %(elem, label)



import unittest
from zope.testing import doctest, cleanup
from Products.Five import zcml

import Products.Five
import kss.core
import kss.core.tests
import plone.app.kss
import plone.app.form
import plone.memoize

optionflags =  (doctest.ELLIPSIS |
                doctest.NORMALIZE_WHITESPACE)


def setUp(test):
    zcml.load_config('configure.zcml', Products.Five)
    zcml.load_config('meta.zcml', kss.core)
    zcml.load_config('configure.zcml', kss.core)
    zcml.load_config('configure-unittest.zcml', kss.core.tests)
    zcml.load_config('configure.zcml', plone.app.form)
    zcml.load_config('configure.zcml', plone.memoize)
    zcml.load_config('configure.zcml', plone.app.kss)

def tearDown(test):
    cleanup.cleanUp()

def test_suite():
    return unittest.TestSuite([
        doctest.DocFileSuite('kss/formlib_kss.txt',
                             package='plone.app.form',
                             setUp=setUp,
                             tearDown=tearDown,
                             optionflags=optionflags),
        ])



from zope.interface import Interface

class IDateComponents(Interface):
    """A view that provides some helper methods useful in date widgets.
    """
    
    def result(date=None,
               use_ampm=0,
               starting_year=None,
               ending_year=None,
               future_years=None,
               minute_step=5):
        """Returns a dict with date information.
        """




import logging
from Products.Five.formlib import formbase

def apply_patches():

    # Five's base class defines template as a ViewPageTemplateFile.
    # However, the base 'zope.formlib.form.FormBase' has 'template'
    # defined as a NamedTemplate, which is much nicer, and what we
    # want.
    if 'template' in formbase.FiveFormlibMixin.__dict__:
        logger = logging.getLogger('plone.app.form')
        logger.debug('*** MONKEYPATCH *** : delete "template" attribute of ' +
           'FiveFormlibMixin to allow use of named templates.')
        delattr(formbase.FiveFormlibMixin, 'template')
